#! /usr/bin/env python
# coding=utf-8

import re
import os
import os.path

from tornado import web
from tornado import ioloop
from tornado.options import define, options
from vanellope import config
from vanellope.database import create_tables, get_admin_user, connection
from vanellope.urls import routers

define("port", default=8000, help="run on the given port", type=int)
define("debug", default=False, help="debug mode.", type=bool)

ROOT = os.path.abspath(os.path.dirname(__file__))


class App(web.Application):
    def __init__(self):

        create_tables()

        theme = config.theme
        static_path = os.path.join(ROOT, 'themes/%s/static' % theme)
        template_path = os.path.join(ROOT, 'themes/%s/templates' % theme)
        admin_static_path = os.path.join(ROOT, 'admin/static')
        admin_template_path = os.path.join(ROOT, 'admin/templates')
        config_keys = config.app_settings.keys()

        settings = {
            "static_path": static_path,
            "template_path": template_path,
            "admin_static_path": admin_static_path,
            "admin_template_path": admin_template_path,
            "static_url_prefix": "/static/",
            "admin_static_url_prefix": "/admin-static/",
            "config_keys": config_keys,
            "uploaded_path": config.uploaded_path,
            "db_path": config.db_path,

            # None or dict object
            # Indicating whether or not the site has a registered admin user
            "admin": get_admin_user(),
            "theme": theme,

            "include_version": True,
            "login_url": '/login',
            "debug": options.debug,
            "db_conn": connection
        }

        if settings.get("admin_static_path"):
            path = settings["admin_static_path"]
            static_url_prefix = settings.get("admin_static_url_prefix",
                                             "/static/")
            static_handler_class = settings.get("static_handler_class",
                                                web.StaticFileHandler)
            static_handler_args = settings.get("admin_static_handler_args", {})
            static_handler_args['path'] = path
            for pattern in [re.escape(static_url_prefix) + r"(.*)",
                            r"/(favicon\.ico)", r"/(robots\.txt)"]:
                routers.insert(0, (pattern, static_handler_class,
                                   static_handler_args))

        web.Application.__init__(self, routers, **settings)

    def log_request(self, handler):
        pass


def make_app():
    return App()


def main():
    options.parse_command_line()
    App().listen(options.port, '127.0.0.1', xheaders=True)
    print 'VANELLOPE running on %d' % options.port
    ioloop.IOLoop.instance().start()


if __name__ == "__main__":
    main()



# coding: utf-8

"""
Shim to allow python -m test.
This only works in python 2.7+.
"""
from __future__ import absolute_import, division, print_function, with_statement

from vanellope.test.runtests import all, main

# vanellope.test.main autodiscovery relies on 'all' being present in
# the main module, so import it here even though it is not used directly.
# The following line prevents a pyflakes warning.
all = all

main()



# coding=utf-8

import os.path
import uuid
from tornado.web import authenticated
from tornado.escape import json_decode
from vanellope.handlers import BaseHandler


class ArticleHandler(BaseHandler):
    def get(self, article_id):
        article = self.posts.find_by_id(article_id)

        if not article:
            self.send_error(404)
            return

        if 'tags' not in article:
            article['tags'] = []

        article['created_at'] = article['created_at'].strftime('%s')  # seconds
        article['updated_at'] = article['updated_at'].strftime('%s')  # seconds
        self.finish(article)


class ConfigurationHandler(BaseHandler):
    @authenticated
    def get(self):
        configs = self.config.read_config()
        self.finish({
            'msg': 'success',
            'configs': configs
        })

    def put(self):
        """ 更新
        """

        configs = {}
        for k in self.settings['config_keys']:
            v = self.get_payload_argument(k, None)
            if v is not None:
                configs[k] = v

        self.config.update(configs)
        self.finish({
            'info': 'success'
        })


class ImageHandler(BaseHandler):
    @authenticated
    def post(self):
        """Upload image
        """

        ifsuccess = self.get_argument('ifsuccess', {})
        # iffail = self.get_argument('iffail', {})
        pathKey = self.get_argument('pathKey', 'url')

        image = self.request.files['image'][0]
        ext = image['filename'].split('.')[-1]

        newFilename = str(uuid.uuid4()) + '.' + ext
        fpath = os.path.join(self.settings['uploaded_path'], newFilename)
        url = '/uploaded/' + newFilename
        with open(fpath, 'wb') as f:
            f.write(image['body'])
            f.close()

        if not isinstance(ifsuccess, dict):
            ifsuccess = json_decode(ifsuccess)

        ifsuccess.update({
            pathKey: url
        })

        self.finish(ifsuccess)


class PostsHandler(BaseHandler):
    def get(self):
        self.finish()

    def post(self):
        """
        创建

        提交参数:
            content
            title
        """
        category = self.get_payload_argument('category', '')
        content = self.get_payload_argument('content', '')
        tags = self.get_payload_argument('tags', '')
        title = self.get_payload_argument('title', 'default_title')
        state = self.get_payload_argument('state', 'draft')
        ext = 'html'

        post_uuid = self.posts.create({
            'content': content,
            'title': title,
            'category': category,
            'tags': tags,
            'ext': ext,
            'state': state
        })

        article_url = '/article/' + post_uuid + '+' + "_".join(title.split())

        self.finish({
            'info': 'success',
            'url': article_url
        })

    def put(self):
        """
        更新

        提交参数:
            content
            title
        """
        category = self.get_payload_argument('category', '')
        content = self.get_payload_argument('content', '')
        uuid = self.get_payload_argument('uuid', '')
        tags = self.get_payload_argument('tags', '')
        title = self.get_payload_argument('title', 'default_title')
        state = self.get_payload_argument('state', 'published')
        ext = 'html'

        self.posts.update(uuid, {
            'uuid': uuid,
            'content': content,
            'title': title,
            'category': category,
            'tags': tags,
            'ext': ext,
            'state': state
        })

        article_url = '/article/' + uuid + '+' + "_".join(title.split())

        self.finish({
            'info': 'success',
            'url': article_url
        })


class PostHandler(BaseHandler):
    @authenticated
    def delete(self, entry_id):
        self.posts.delete(entry_id)
        self.finish({
            'msg': 'success'
        })


class AdminTrashHandler(BaseHandler):
    @authenticated
    def delete(self, entry_id):
        self.posts.drop(entry_id)
        self.finish({
            'msg': 'success'
        })



# coding=utf-8

from vanellope.handlers import BaseHandler


class MainFeed(BaseHandler):
    def get(self):
        # 每页显示 N 篇文章

        current_page = int(self.get_argument(u'p', 1))

        _posts = self.posts.get_posts()

        entries = []

        for article in _posts:
            if 'tags' not in article:
                article['tags'] = []

            article['short_path'] = '/article/' + article['uuid']
            article['long_path'] = ''.join('/article/',
                                           article['uuid'],
                                           u'_'.join(article['title'].split()))
            article['edit_path'] = '/admin/edit/' + article['long_path']
            entries.append(article)

        self.set_header('Content-Type', 'application/atom+xml; charset=UTF-8')

        self.render("feeds.xml",
                    title=u'首页 | VANELLOPE',
                    page=u'index',
                    previous_page=current_page - 1 if current_page > 1 else 1,
                    next_page=current_page + 1,
                    entries=entries,
                    articles=entries)



# coding=utf-8

import math
import hashlib
import base64
from tornado import web
from tornado.web import authenticated
from vanellope.handlers import BaseHandler
from vanellope import config
from vanellope.handlers import Days


class WelcomePage(BaseHandler):
    def get(self):

        admin = self.user.get_admin_user()

        if admin:
            return self.redirect('/')

        self.render("welcome.html",
                    title=u'Welcome',
                    page=u'welcome')

    def post(self):
        """Signup"""

        admin = self.user.get_admin_user()

        if admin:
            self.set_status(400)
            return self.finish({
                'status': u'error'
            })

        email = self.get_argument('email', None)
        pwd = self.get_argument('pwd', None)
        role = self.get_argument('role', None)

        err = 0
        msg = u'login successfully'

        admin_user = self.user.get_admin_user()
        if role == 'admin' and admin_user:
            err += 1
            msg = u'Admin user exists'

        else:
            password_hash = self.user.create_user({
                "username": "Admin",
                "email": email,
                "password": pwd,
                "role": role
            })

            self.clear_all_cookies()
            cookie = base64.b64encode('Admin:' + password_hash)
            self.set_cookie(name="vanellope",
                            value=cookie,
                            expires_days=90)

            admin_user = self.user.get_admin_user()
            self.settings['admin'] = admin_user

        self.finish({
            'status': u'success',
            'msg': msg
        })


class IndexPage(BaseHandler):
    def get(self):

        ENTRIES_PER_PAGE = config.posts_per_page
        current_page = int(self.get_argument(u'p', 1))

        current_user = self.get_current_user()
        if current_user and current_user['role'] == 'admin':
            drafts = self.posts.get_drafts()
        else:
            drafts = []

        articles = self.posts.find(
            states=['published'],
            limit=ENTRIES_PER_PAGE,
            skip=(current_page - 1) * ENTRIES_PER_PAGE
        )

        total_entries = self.posts.count(states=['published'])

        pages = int(math.floor(total_entries / ENTRIES_PER_PAGE))
        if total_entries > pages * ENTRIES_PER_PAGE:
            pages += 1

        self.render("index.html",
                    title=u'首页 | VANELLOPE',
                    page=u'index',
                    previous_page=current_page - 1 if current_page > 1 else 1,
                    next_page=current_page + 1,
                    pages=pages,
                    drafts=drafts,
                    articles=articles)


class TagsPage(BaseHandler):
    def get(self):
        current_page = int(self.get_argument(u'p', 1))

        self.render("tags.html",
                    title=u'标签 | VANELLOPE',
                    page=u'tags',
                    previous_page=current_page - 1 if current_page > 1 else 1,
                    next_page=current_page + 1)


class TagPage(BaseHandler):
    def get(self, tag):
        current_page = int(self.get_argument(u'p', 1))

        articles = self.posts.find_posts_with_tag(tag)

        self.render("tag.html",
                    title=u"VANELLOPE | Tag:{0}".format(tag),
                    page=u'tag',
                    previous_page=current_page - 1 if current_page > 1 else 1,
                    next_page=current_page + 1,
                    current_tag=tag,
                    articles=articles)


class ArchivesPage(BaseHandler):
    def get(self):
        self.render('archives.html',
                    title=u'归档 | VANELLOPE',
                    page=u'archives')


class ArchivePage(BaseHandler):
    def get(self, year=u'', month=u'', day=u'12345'):
        if day is None:
            day = u''

        if month is None:
            month = u''

        if year is None:
            year = u''

        current_page = int(self.get_argument(u'p', 1))

        from_date = end_date = None

        if year and month and day:
            from_date = (
                Days()
                .day(u"{0}-{1}-{2}".format(year, month, day))
                .timezone('UTC')
                .beginning()
            )

            end_date = (
                Days()
                .day(u"{0}-{1}-{2}".format(year, month, day))
                .timezone('UTC')
                .next_day()
                .beginning()
            )

        elif year and month and not day:
            from_date = (
                Days()
                .day(u"{0}-{1}-{2}".format(year, month, u'01'))
                .timezone('UTC')
                .beginning()
            )

            end_date = (
                Days()
                .day(u"{0}-{1}-{2}".format(year, month, u'01'))
                .timezone('UTC')
                .next_month()
                .beginning()
            )

        elif year and not month and day:
            from_date = (
                Days()
                .day(u"{0}-{1}-{2}".format(year, u'01', day))
                .timezone('UTC')
                .beginning()
            )

            end_date = (
                Days()
                .day(u"{0}-{1}-{2}".format(year, u'12', day))
                .timezone('UTC')
                .beginning()
            )

        elif year and not month and not day:
            from_date = (
                Days().day(u"{0}-01-01".format(year))
                .timezone('UTC')
                .beginning()
            )

            end_date = (
                Days().day(u"{0}-12-31".format(year))
                .timezone('UTC')
                .beginning()
            )

        elif not year:
            self.redirect('/')

        posts = self.posts.find_posts_between_date(from_date, end_date)

        articles = []

        for article in posts:
            if 'tags' not in article:
                article['tags'] = []
            articles.append(article)

        self.render("archive.html",
                    title=u'归档 | VANELLOPE',
                    page=u'archive',
                    from_date=from_date,
                    end_date=end_date,
                    current_uri=self.request.uri,
                    previous_page=current_page - 1 if current_page > 1 else 1,
                    next_page=current_page + 1,
                    articles=articles)


class CategoryPage(BaseHandler):
    def get(self, cate):
        current_page = int(self.get_argument(u'p', 1))
        articles = self.posts.find_by_category(cate)

        self.render("category.html",
                    title=u'Category:{0} | VANELLOPE'.format(cate),
                    page=u'category',
                    previous_page=current_page - 1 if current_page > 1 else 1,
                    next_page=current_page + 1,
                    current_category=cate,
                    articles=articles)


class LoginPage(BaseHandler):
    def get(self):
        self.render("login.html",
                    title=u'登录 | VANELLOPE',
                    page=u'login')

    def post(self):
        """Login"""

        pwd = self.get_argument('pwd', None)

        err = 0

        admin_user = self.user.get_admin_user()

        if not admin_user:
            return self.redirect('/welcome')

        new_hash = hashlib.sha256(pwd + admin_user['salt']).hexdigest()
        if admin_user['passwd'] == new_hash:
            self.clear_all_cookies()
            cookie = base64.b64encode('Admin:' + new_hash)
            self.set_cookie(name="vanellope",
                            value=cookie,
                            expires_days=90)
        else:
            err += 1

        if err == 0:
            self.redirect('/')

        else:
            self.redirect('/login')


class Logout(BaseHandler):
    def get(self):
        self.clear_all_cookies()
        self.redirect('/')


class DraftPage(BaseHandler):
    @authenticated
    def get(self, article_id):
        article = self.posts.find_by_id(article_id)

        if not article:
            self.send_error(404)
            return

        if 'tags' not in article:
            article['tags'] = []

        siblings = []

        self.render("article.html",
                    title=article['title'],
                    page=u'draft',
                    related_articles=[],
                    siblings=siblings,
                    article=article)


class ArticlePage(BaseHandler):
    def get(self, article_id):
        try:
            article = self.posts.find(id_list=[article_id],
                                      states=['published'])[0]
        except IndexError:
            self.send_error(404)
            return

        if 'tags' not in article:
            article['tags'] = []

        siblings = []

        self.render("article.html",
                    title=article['title'],
                    page=u'article',
                    related_articles=[],
                    siblings=siblings,
                    article=article)

    def post(self):
        """Create new article"""
        pass

    @authenticated
    def put(self, article_id=None):
        """ Update an existing article """
        pass

    @authenticated
    def delete(self, article_id):
        """ Delete article """
        pass


class UploadedFileHandler(web.StaticFileHandler):
    def initialize(self):
        uploaded_path = self.settings['uploaded_path']
        return web.StaticFileHandler.initialize(self, uploaded_path)



#! /usr/bin/env python
# coding=utf-8

"""
Default configurations.
Those can be overide by user specify settings
"""

import os
import os.path

# The theme name
theme = 'default'

# How many post displayed on one page
posts_per_page = 10

# SQLite3 Settings
db_path = os.path.join(os.getcwd(), u'content/data.db')

if not os.path.exists(os.path.dirname(db_path)):
    os.makedirs(os.path.dirname(db_path))

# 数据库备份目录
backup_path = os.path.join(os.getcwd(), u'content/backups/')
if not os.path.exists(os.path.dirname(backup_path)):
    os.makedirs(os.path.dirname(backup_path))

# 上传文件的存储路径
# 绝对路径
uploaded_path = os.path.join(os.getcwd(), u'content/www/')

if not os.path.exists(os.path.dirname(uploaded_path)):
    os.makedirs(os.path.dirname(uploaded_path))

create_table_sqls = {
    "posts_schema": """
                    CREATE TABLE IF NOT EXISTS posts (
                         uuid TEXT PRIMARY KEY,
                         ext TEXT,
                         title TEXT,
                         content TEXT,
                         category TEXT NOT NULL,
                         tags TEXT,
                         state TEXT NOT NULL DEFAULT 'published',
                         created_at TIMESTAMP NOT NULL,
                         updated_at TIMESTAMP NOT NULL
                    )
                    """,

    "users_schema": """
                    CREATE TABLE IF NOT EXISTS users (
                        username TEXT PRIMARY KEY,
                        email TEXT UNIQUE,
                        passwd TEXT,
                        salt TEXT,
                        role TEXT,
                        created_at NOT NULL DEFAULT CURRENT_TIMESTAMP,
                        updated_at NOT NULL DEFAULT CURRENT_TIMESTAMP
                    )
                    """,

    "conf_schema": """
                    CREATE TABLE IF NOT EXISTS configuration (
                        key TEXT PRIMARY KEY,
                        value TEXT
                    )
                    """
}

app_settings = {
    # 网站标题
    'site_title': 'Life with vanellope',

    # 网站地址 like 「www.example.com」
    'site_url': '',

    # 网站统计服务
    'site_tracking': '', # 'google_analystics'
    'site_tracking_id': '',
    'site_tracking_enabled': 'yes',

    # 网站评论服务
    'site_comment': '' , # 'duoshuo' or 'disqus'
    'site_comment_id': '' ,
    'site_comment_enabled': 'no',

    # Post default category
    # 新建文章的默认分类
    'default_category': 'life',

    # draft url: http://www.example.com/drafts/<uuid>+title
    'draft_base_path': '/drafts/'
}



#!/usr/bin/env python
# coding: utf-8

from urlparse import urlparse
from tornado import testing
from tornado import httpclient
from vanellope import config
from vanellope import app
from tornado import ioloop

# http://www.tornadoweb.org/en/stable/testing.html#tornado.testing.AsyncHTTPTestCase

class TestPages(testing.AsyncHTTPTestCase):
    def get_app(self):
        return app.make_app()

    def test_login_page(self):
        target_url = '/login'
        response = self.fetch(target_url)
        parsed_url = urlparse(response.effective_url)
        self.assertEqual(response.code, 200)
        self.assertEqual(parsed_url.path, target_url)

    def test_logout_page(self):
        target_url = '/logout'
        response = self.fetch(target_url)
        parsed_url = urlparse(response.effective_url)
        self.assertEqual(response.code, 200)
        self.assertEqual(parsed_url.path, '/')

    def test_index_page(self):
        target_url = '/'
        response = self.fetch(target_url)
        parsed_url = urlparse(response.effective_url)
        self.assertEqual(response.code, 200)
        self.assertEqual(parsed_url.path, target_url)

    def test_article_page(self):
        target_url = '/article/32ea9174'
        response = self.fetch(target_url)
        parsed_url = urlparse(response.effective_url)
        self.assertEqual(response.code, 200)
        self.assertEqual(parsed_url.path, target_url)

    def test_tags_page(self):
        target_url = '/tags'
        response = self.fetch(target_url)
        parsed_url = urlparse(response.effective_url)
        self.assertEqual(response.code, 200)
        self.assertEqual(parsed_url.path, target_url)

    def test_tag_page(self):
        target_url = '/tags/vanellope'
        response = self.fetch(target_url)
        parsed_url = urlparse(response.effective_url)
        self.assertEqual(response.code, 200)
        self.assertEqual(parsed_url.path, target_url)

    def test_archives_page(self):
        target_url = '/archives'
        response = self.fetch(target_url)
        parsed_url = urlparse(response.effective_url)
        self.assertEqual(response.code, 200)
        self.assertEqual(parsed_url.path, target_url)

    def test_category_page(self):
        target_url = '/category/vanellope'
        response = self.fetch(target_url)
        parsed_url = urlparse(response.effective_url)
        self.assertEqual(response.code, 200)
        self.assertEqual(parsed_url.path, target_url)

    def test_archive_page(self):
        pass


    def test_admin_page(self):
        # http://www.tornadoweb.org/en/stable/httpclient.html#tornado.httpclient.HTTPResponse

        target_url = '/admin'
        response = self.fetch(target_url,
            follow_redirects = False
        )
        parsed_url = urlparse(response.effective_url)
        self.assertEqual(response.code, 302)
        self.assertEqual(parsed_url.path, target_url)


if __name__ == '__main__':
    unittest.main()



# coding=utf-8

import urllib
import string
import random
import hashlib
import re
import pytz

import itertools
from datetime import datetime, timedelta
from dateutil import relativedelta

import base64
from tornado.web import StaticFileHandler, RequestHandler, MissingArgumentError
from tornado.log import access_log
from tornado.escape import json_decode

from user_agents import parse as ua_parse
import urlparse

from vanellope.database import ConfigModel
from vanellope.database import UserModel
from vanellope.database import PostModel
from vanellope.database import Session


class Days(object):
    TIMEZONES = list(itertools.chain(*pytz.country_timezones.values()))
    timezone = None

    def day(self, day, tz="UTC"):
        """ when we accept the day string, we assume it's in local timezone"""
        # if the 'day' parameter is not specified, return today's beginning
        if (day and isinstance(day, basestring) and
           re.match('^\d{4}-\d{2}-\d{2}$', day)):

            self.day = datetime.strptime(day, '%Y-%m-%d')
        else:
            x = datetime.today().replace(tzinfo=pytz.timezone(tz))
            day = (x + x.utcoffset()).strftime('%Y-%m-%d')
            self.day = datetime.strptime(day, '%Y-%m-%d')

        return self

    def timezone(self, tz):
        # tz parameter maybe invalid
        if tz is not None and tz not in self.TIMEZONES and tz != 'UTC':
            self.timezone = ""
        else:
            self.timezone = tz

        return self

    def beginning(self):
        return self.day.replace(
            hour=0,
            minute=0,
            second=0,
            microsecond=0,
            tzinfo=pytz.timezone(self.timezone)
        )

    def next_day(self):
        self_fake = self
        self_fake.day = self.day + timedelta(days=1)
        return self_fake

    def next_month(self):
        self_fake = self
        self_fake.day = self.day + relativedelta.relativedelta(months=1)
        return self_fake

    def next_year(self):
        self_fake = self
        self_fake.day = self.day + relativedelta.relativedelta(years=1)
        return self_fake


class BaseHandler(RequestHandler):
    user = UserModel()
    posts = PostModel()
    config = ConfigModel()
    session = Session()

    def prepare(self):
        try:
            user_agent = self.request.headers['User-Agent']
        except KeyError:
            user_agent = '-'

        access_log.info('request log -> %s|%s|%s|%s' % (
            self.request.remote_ip,
            self.request.method,
            self.request.uri,
            user_agent)
        )

        if not self.settings['admin'] and self.request.uri != '/welcome':
            self.redirect('/welcome')

    def get_template_namespace(self):
        """Override `tornado.web.RequestHandler.get_template_namespace` static method
        add those new features:

        """

        namespace = super(BaseHandler, self).get_template_namespace()
        namespace['site'] = self.config.read_config()

        try:
            user_agent = self.request.headers['User-Agent']
        except KeyError:
            user_agent = '-'

        namespace['req'] = {

            # 当前请求路径
            'path': urlparse.urlparse(self.request.uri).path,

            'hostname': self.request.protocol + '://' + self.request.host,

            # userAgent 实例
            'user_agent': ua_parse(user_agent)
        }

        namespace['archives'] = self.posts.get_archives()
        namespace['tags'] = self.posts.get_tags()
        namespace['categories'] = []

        return namespace

    def get_payload_argument(self, name, default, strip=True):
        """ Get request data from payload

        """
        source = json_decode(self.request.body)
        if isinstance(source, dict):
            if name in source:
                arg = source[name]
                if isinstance(arg, basestring):
                    arg = self.decode_argument(arg)
                    # Get rid of any weird control chars (unless decoding gave
                    # us bytes, in which case leave it alone)
                    arg = RequestHandler._remove_control_chars_regex.sub(" ", arg)
                    if strip:
                         arg = arg.strip()

                elif isinstance(arg, list):
                    arg = filter(lambda item: self.decode_argument(item).strip(), arg)
                return arg
            else:
                return default

        raise MissingArgumentError(name)

    def gravatar(self, email, size=64):
        gravatar_url = ("http://www.gravatar.com/avatar/%s" %
                        hashlib.md5(email).hexdigest() + "?")
        gravatar_url += urllib.urlencode({'s': str(size)})
        return gravatar_url

    def randomwords(self, length):
        random.seed()
        chars = string.lowercase + string.uppercase + string.digits
        return ''.join(random.choice(chars) for i in range(length))

    def get_hash_string(self, origin_string):
        return hashlib.sha256(origin_string).hexdigest()

    def is_number(self, arg):
        return isinstance(arg, int)

    def get_current_user(self):
        auth = self.get_cookie('vanellope')
        if not auth:
            return

        try:
            username, passwd_hash = base64.b64decode(auth).split(':')
        except:
            return None

        if not username:
            return None
        else:
            user = self.user.get_admin_user()
            return user

    def striphtml(self, data):
        p = re.compile(r'<.*?>')
        return p.sub('', data)


class AdminBaseHandler(BaseHandler):
    def static_url(self, path, include_host=None, **kwargs):
        """Override original static_url method to server admin static files
        """
        self.require_setting("admin_static_path", "static_url")
        get_url = self.settings.get("static_handler_class",
                                    StaticFileHandler).make_static_url

        if include_host is None:
            include_host = getattr(self, "include_host", False)

        if include_host:
            base = self.request.protocol + "://" + self.request.host
        else:
            base = ""

        settings = dict(
            static_url_prefix=self.settings['admin_static_url_prefix'],
            static_path=self.settings['admin_static_path']
        )
        return base + get_url(settings, path, **kwargs)

    def get_template_path(self):
        """Overide default self.get_template_path method.

        In order to render admin page independently
        """
        return self.application.settings.get("admin_template_path")



# coding=utf-8

from tornado.web import authenticated
from vanellope.handlers import AdminBaseHandler
from vanellope.database import db_backup


class AdminSettingsPage(AdminBaseHandler):
    @authenticated
    def get(self):
        self.render("admin.html",
                    title=u'控制台 | VANELLOPE',
                    page=u'admin/settings')


class AdminEditPage(AdminBaseHandler):
    @authenticated
    def get(self, article_id=None):
        article = self.posts.find_by_id(article_id)

        if article_id and article is None:
            return self.send_error(404)

        self.render("write.html",
                    title=u'编辑 | VANELLOPE',
                    page=u'admin/edit',
                    article=article)


class AdminDraftsPage(AdminBaseHandler):
    @authenticated
    def get(self):
        articles = self.posts.get_drafts()

        self.render("drafts.html",
                    title=u'草稿箱 | VANELLOPE',
                    page=u'admin/drafts',
                    articles=articles)


class AdminTrashPage(AdminBaseHandler):
    @authenticated
    def get(self):
        articles = self.posts.get_trash()

        self.render("trash.html",
                    title=u'垃圾箱 | VANELLOPE',
                    page=u'admin/trash',
                    articles=articles)


class AdminExportData(AdminBaseHandler):
    def get(self):
        """Export database
        """
        zip_path, zip_filename = db_backup()

        with open(zip_path, 'rb') as f:
            self.set_header('Content-Type', 'application/octet-stream')
            self.set_header('Content-Disposition',
                            'attachment; filename=' + zip_filename)
            self.write(f.read())
            self.finish()



# coding=utf-8

import os
import os.path
import calendar
import zipfile
import datetime
import random
import sqlite3
import string
import hashlib
import time
from vanellope import config
from tornado.util import ObjectDict


# Autocommit ON
connection = sqlite3.connect(
    config.db_path,
    isolation_level=None,
    detect_types=sqlite3.PARSE_DECLTYPES
)
session_connection = sqlite3.connect(':memory:')


def get_admin_user():
    return UserModel().get_admin_user()


def create_tables():
    # Create tables if not exist
    cur = connection.cursor()

    # Create configuration table
    cur.execute(config.create_table_sqls['conf_schema'])
    cur.execute('SELECT * FROM configuration')
    results = cur.fetchall()

    # Initialize configurations
    if len(results) == 0:
        t = config.app_settings.items
        cur.executemany("INSERT INTO configuration VALUES (?, ?)", t)

    # Create posts table
    cur.execute(config.create_table_sqls['posts_schema'])

    # Create users schema
    cur.execute(config.create_table_sqls['users_schema'])


def db_backup():
    current_time_str = datetime.datetime.now().strftime('%Y-%m-%dT%H-%M-%S%ZZ')
    filename = '.'.join([current_time_str, 'sql'])
    full_path = os.path.join(config.backup_path, filename)
    dump = os.linesep.join(connection.iterdump())

    zip_path = full_path + '.zip'
    zip_filename = filename + '.zip'

    zf = zipfile.ZipFile(zip_path, 'w', zipfile.ZIP_DEFLATED)
    zf.writestr(filename, dump.encode('utf-8'))
    zf.close()

    return zip_path, zip_filename


def random_string(n):
    string_set = string.ascii_letters + string.digits + "!@#$%^&*<>?{}[]"
    return ''.join(random.choice(string_set) for _ in range(n))


class DataAccess(object):
    """读取数据库的通用方法集
    """

    # Get Database Connection
    @property
    def conn(self):
        return connection

    def to_dict(self, cur, row):
        """Convert SQLite3 results to dict

           http://stackoverflow.com/questions/3300464
        """
        if not row:
            return

        d = {}
        for idx, col in enumerate(cur.description):
            d[col[0]] = row[idx]
        return d

    def get_admin(self):
        """Get master user
        """

        cur = self.conn.cursor()
        cur.execute("""
            SELECT * from users WHERE role = "master";
        """)
        master = cur.fetchone()
        return master


class ConfigModel(DataAccess):
    """ Site configuration
    """

    # 网站的全部配置
    # 使用 Tornado 提供的 ObjectDict，
    # 可以使用 . 运算符读取属性
    config = ObjectDict()

    def __init__(self):
        super(DataAccess, self).__init__()

    def read_config(self):
        cur = self.conn.cursor()
        results = cur.execute("SELECT * FROM configuration").fetchall()

        configs = {}
        if results:
            results = [self.to_dict(cur, item) for item in results]

        for r in results:
            configs[r['key']] = r['value']

        return configs

    def set_value(self, k, v):
        sql = """
            UPDATE config SET value = ? WHERE key = ?
        """
        cur = self.conn.cursor()
        cur.execute(sql, (v, k))
        self.conn.commit()
        return

    def get_value(self, k):
        sql = """
            SELECT value FROM config WHERE key = ?
        """

        cur = self.conn.cursor()
        cur.execute(sql, (k,))
        return cur.fetchone()

    @property
    def site_name(self):
        site_name = self.get_value('site_name')
        return site_name[0]

    def update(self, data):
        cur = self.conn.cursor()
        t = tuple((v, k) for k,v in data.items())
        cur.executemany("UPDATE configuration SET value = ? WHERE key = ?", t)


class UserModel(DataAccess):
    def __init__(self):
        pass

    def create(self, data):
        cur = self.conn.cursor()
        sql = """
            INSERT INTO users
            (username, passwd, salt, role)
            VALUES (?, ?, ?, ?)
        """

        salt = random_string(8)
        passwd_hash = hashlib.sha256(data['password'] + salt).hexdigest()

        cur.execute(sql, (data['username'], passwd_hash, salt, data['role']))
        self.conn.commit()
        return passwd_hash

    def create_user(self, data):
        cur = self.conn.cursor()
        sql = """
            INSERT INTO users
            (username, email, passwd, salt, role)
            VALUES (?, ?, ?, ?, ?)
        """

        salt = random_string(8)
        passwd_hash = hashlib.sha256(data['password'] + salt).hexdigest()

        cur.execute(sql, (
            data['username'],
            data['email'],
            passwd_hash,
            salt,
            data['role']
        ))
        self.conn.commit()
        return passwd_hash

    def get_admin_user(self):
        cur = self.conn.cursor()
        cur.execute("""
            SELECT * FROM users WHERE role="admin" AND username="Admin"
        """)

        admin_user = self.to_dict(cur, cur.fetchone())
        if admin_user is not None:
            admin_user = ObjectDict(admin_user)

        return admin_user


class PostModel(DataAccess):
    """
    """

    __post = None

    def __parse(self, item):
        if 'tags' in item:
            tags = item['tags'].split(',')
            item['tags'] = filter(None, [t.strip() for t in tags])

        if item['state'] == 'draft':
            item['path'] = ''.join([
                config.app_settings['draft_base_path'],
                item['uuid'],
                '+',
                u'_'.join(item['title'].split())
            ])

        elif item['state'] == 'published':
            item['path'] = ''.join([
                '/article/',
                item['uuid'],
                '+',
                u'_'.join(item['title'].split())
            ])

        elif item['state'] == 'deleted':
            item['path'] = ''.join([
                '/trash/',
                item['uuid'],
                '+',
                u'_'.join(item['title'].split())
            ])

        item['editor-path'] = ''.join([
            '/admin/edit/',
            item['uuid'],
            '+',
            u'_'.join(item['title'].split())
        ])

        return item

    def create(self, data):
        """Create new post
        """

        if type(data) is not dict:
            raise TypeError('parameter `data` must be a dict object')

        params = []

        assert data['ext']
        assert data['ext'] == 'html'
        params.append(data['ext'])

        assert data['title'] and len(data['title']) > 0
        params.append(data['title'])

        assert data['content'] and len(data['content']) > 0
        params.append(data['content'])

        if 'category' in data:
            category = data['category']
        else:
            category = config.app_settings['default_category']

        params.append(category)

        tags_str = ','.join(data['tags']) if 'tags' in data else ''
        params.append(','.join(['', tags_str, '']))
        params.append(data['state'])

        cur = self.conn.cursor()

        sql = """ INSERT INTO posts
                  (ext, title, content, category, tags,
                   state, uuid, created_at, updated_at)
                  VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)
              """

        post_id = self.gen_uuid(data['title'], data['content'])
        params.append(post_id)
        params.append(datetime.datetime.now())
        params.append(datetime.datetime.now())

        cur.execute(sql, params)
        self.conn.commit()

        return post_id

    def update(self, uid, data):
        """Update an article by it's id
        """

        if type(data) is not dict:
            raise TypeError('parameter `data` must be a dict object')

        params = []

        assert data['ext']
        assert data['ext'] == 'html'
        params.append(data['ext'])

        assert data['title'] and len(data['title']) > 0
        params.append(data['title'])

        assert data['content'] and len(data['content']) > 0
        params.append(data['content'])

        if 'category' in data:
            category = data['category']
        else:
            category = config.app_settings['default_category']

        params.append(category)

        tags_str = ','.join(data['tags']) if 'tags' in data else ''
        params.append(','.join(['', tags_str, '']))

        params.append(data['state'])

        cur = self.conn.cursor()

        sql = """
              UPDATE posts
              SET ext = ?,
                  title = ?,
                  content = ?,
                  category = ?,
                  tags = ?,
                  state = ?,
                  updated_at = ?
              WHERE uuid = ?
              """
        params.append(datetime.datetime.now())
        params.append(uid)

        cur.execute(sql, params)
        self.conn.commit()

    def find(self, id_list=[], tag_list=[], limit=None, skip=0,
             before_date=None, after_date=None, states=[], categories=[]):

        id_list = filter(None, id_list)
        tag_list = filter(None, tag_list)
        states = filter(None, states)
        categories = filter(None, categories)

        sql = """
              SELECT

                  uuid,
                  ext,
                  title,
                  content,
                  category,
                  tags,
                  state,
                  created_at as "created_at [timestamp]",
                  updated_at

              FROM posts WHERE 1
              """
        params = []

        if len(id_list) > 0:
            sql += " AND uuid IN (?) "
            params.append(','.join(id_list))

        if len(tag_list) > 0:
            sql += " AND tags LIKE "
            sql += ' OR '.join(['?'] * len(tag_list))
            params += ['%' + tag + '%' for tag in tag_list]

        if before_date:
            sql += " AND created_at <= ?"
            params.append(before_date)

        if after_date:
            sql += " AND created_at >= ?"
            params.append(after_date)

        if len(states) > 0:
            sql += " AND state IN (?)"
            params.append(','.join(states))

        if len(categories) > 0:
            sql += " AND category IN (?)"
            params.append(','.join(categories))

        sql += " ORDER BY created_at DESC"

        if limit:
            sql += " LIMIT ?"
            params.append(limit)

        if skip:
            sql += " OFFSET ?"
            params.append(skip)

        cur = self.conn.cursor()
        cur.execute(sql, params)
        return [self.__parse(self.to_dict(cur, p)) for p in cur.fetchall()]

    def count(self,
              tag_list=[],
              before_date=None,
              after_date=None,
              states=[],
              categories=[]):

        tag_list = filter(None, tag_list)
        states = filter(None, states)
        categories = filter(None, categories)

        sql = " SELECT count(*) as count FROM posts WHERE 1 "
        params = []

        if len(tag_list) > 0:
            sql += " AND tags LIKE "
            sql += ' OR '.join(['?'] * len(tag_list))
            params += ['%' + tag + '%' for tag in tag_list]

        if before_date:
            sql += " AND created_at <= ?"
            params.append(before_date)

        if after_date:
            sql += " AND created_at >= ?"
            params.append(after_date)

        if len(states) > 0:
            sql += " AND state IN (?)"
            params.append(','.join(states))

        if len(categories) > 0:
            sql += " AND category IN (?)"
            params.append(','.join(categories))

        sql += " ORDER BY created_at DESC"

        cur = self.conn.cursor()
        cur.execute(sql, params)
        return cur.fetchone()[0]

    def delete(self, uid):
        cur = self.conn.cursor()

        sql = """
              UPDATE posts
              SET state = ?,
                  updated_at = ?
              WHERE uuid = ?
              """
        params = ['deleted']
        params.append(datetime.datetime.now())
        params.append(uid)

        cur.execute(sql, params)
        self.conn.commit()

    def drop(self, uid):
        """Delete from trash. can't be recovered
        """

        cur = self.conn.cursor()

        sql = """
              DELETE FROM posts
              WHERE state = 'deleted'
              AND uuid = ?
              """
        params = [uid]

        cur.execute(sql, params)
        self.conn.commit()

    def find_posts_with_tag(self, tag):
        return self.find(tag_list=[tag])

    def find_posts_between_date(self, start_date, end_date):
        return self.find(
            states=['published'],
            before_date=end_date,
            after_date=start_date
        )

    def get_archives(self):
        """按创建日期给文章分组,
           并返回结构化的数据
        """
        posts = self.find(states=['published'])

        archives = {}
        for a in posts:
            year = a['created_at'].year
            month = a['created_at'].strftime('%m')
            month_abbr = calendar.month_abbr[a['created_at'].month]

            if year not in archives:
                archives[year] = {}

            if month not in archives[year]:
                archives[year][month] = {
                    'count': 0,
                    'abbr': month_abbr,
                    'posts': []
                }

            archives[year][month]['count'] += 1
            archives[year][month]['posts'].append(a)

        return archives

    def find_by_id(self, uuid):
        """Find an article by it's uuid
        """
        posts = self.find(id_list=[uuid])
        try:
            return posts[0]
        except IndexError:
            return None

    def find_by_category(self, cate):
        """Find posts with specific category"""
        return self.find(categories=[cate])

    def gen_uuid(self, title, content):
        """Generate post uuid based on it's content
        """

        m = hashlib.sha1()
        s = title + content + str(time.time())
        m.update(s.encode('utf-8'))
        uuid = m.hexdigest()[:8]
        return uuid

    def get_drafts(self):
        """Find all posts that are drafts.
        """
        return self.find(states=['draft'])

    def get_trash(self):
        """Find all posts that are deleted
        """
        return self.find(states=['deleted'])

    def find_published_posts(self):
        """Find all posts that are published."""
        return self.find(states=['published'])

    def find_published_posts_by_id(self, id_list):
        """Find all posts that are published."""
        return self.find(states=['published'])

    def get_posts(self):
        return self.find()

    def get_tags(self):
        """Find all tags availabilie
        """

        cur = self.conn.cursor()
        sql = """
        select tags from posts where state = 'published'
        """
        cur.execute(sql)
        article_tags = cur.fetchall()
        flat_tags_list = []

        for tags in article_tags:
            for t in tags[0].split(','):
                flat_tags_list.append(t)

        flat_tags_list = filter(None, [t.strip() for t in flat_tags_list])
        tags_count = {}
        for tag in flat_tags_list:
            if tag not in tags_count:
                tags_count[tag] = 0

            tags_count[tag] += 1

        result = []
        for tag, count in tags_count.items():
            result.append({
                'tag': tag,
                'count': count
            })

        return result


class Session(DataAccess):
    def __init__(self):
        cur = self.conn.cursor()

        cur.execute("""
            CREATE TABLE IF NOT EXISTS user_session (
                auth TEXT PRIMARY KEY,
                username TEXT
            )
        """)

        self.conn.commit()
        cur.close()

    @property
    def conn(self):
        return session_connection

    def set_user_session(self, auth, username):
        cur = self.conn.cursor()
        cur.execute("""
            INSERT INTO user_session
            (auth, username)
            VALUES (?, ?)
        """, (auth, username))
        self.conn.commit()
        cur.close()

    def get_user_session(self, auth):
        cur = self.conn.cursor()
        cur.execute("""
            SELECT * FROM user_session
        """)
        username = cur.fetchall()
        try:
            return username[0]
        except:
            return None



#!/usr/bin/env python
# coding: utf-8

from __future__ import absolute_import, division, print_function, with_statement

import sys
from tornado.testing import unittest

TEST_MODULES = [
    'vanellope.test.database_test',
    'vanellope.test.pages_test',
    'vanellope.test.apiv1_test'
]

def all():
    return unittest.defaultTestLoader.loadTestsFromNames(TEST_MODULES)

class TornadoTextTestRunner(unittest.TextTestRunner):
    def run(self, test):
        result = super(TornadoTextTestRunner, self).run(test)
        if result.skipped:
            skip_reasons = set(reason for (test, reason) in result.skipped)
            self.stream.write(textwrap.fill(
                "Some tests were skipped because: %s" %
                ", ".join(sorted(skip_reasons))))
            self.stream.write("\n")
        return result

def main():
    import tornado.testing

    kwargs = {}
    kwargs['testRunner'] = TornadoTextTestRunner

    try:
        tornado.testing.main(**kwargs)
    finally:
        sys.exit(1)


if __name__ == '__main__':
    main()



# coding=utf-8

import config
from tornado import web

from vanellope.handlers import pages
from vanellope.handlers import apiv1
from vanellope.handlers import admin
from vanellope.handlers import feeds

routers = [
    (r"/welcome", pages.WelcomePage),

    # Index page
    (r"/", pages.IndexPage),

    # 登录
    (r"/authentication", pages.WelcomePage),

    (r"/tags", pages.TagsPage),
    (r"/archives", pages.ArchivesPage),

    # Articles about one specific tag
    (r"/tags/(.*)", pages.TagPage),

    # Show articles written in a specific date range
    # accept date range from 0000-00-00 to 9999-99-99
    (r"/archive/([0-9]{4})/?([0-9]{2})?/?([0-9]{2})?", pages.ArchivePage),

    # Articles with one specific category
    (r"/category/(.*)", pages.CategoryPage),

    # GET 查看某篇文章
    (r"/article/(\w{8}).*", pages.ArticlePage),

    # 查看草稿
    (config.app_settings['draft_base_path'] + r"(\w{8}).*", pages.DraftPage),

    # 登出
    (r"/logout", pages.Logout),
    (r"/login", pages.LoginPage),

    (r"/uploaded/(.*)", pages.UploadedFileHandler),

    (r"/api/v1/posts", apiv1.PostsHandler),
    (r"/api/v1/posts/(.*)", apiv1.PostHandler),
    (r"/api/v1/article/(.*)", apiv1.ArticleHandler),
    (r"/api/v1/image", apiv1.ImageHandler),
    (r"/api/v1/configuration", apiv1.ConfigurationHandler),
    (r"/api/v1/admin/trash/(.*)", apiv1.AdminTrashHandler),

    # Feeds
    (r"/index.xml", feeds.MainFeed),

    # Administrotor URLs
    (r"/admin", admin.AdminSettingsPage),
    (r"/admin/drafts", admin.AdminDraftsPage),
    (r"/admin/trash", admin.AdminTrashPage),
    (r"/admin/settings", admin.AdminSettingsPage),
    (r"/admin/edit", admin.AdminEditPage),
    (r"/admin/edit/(\w{8})\+.*", admin.AdminEditPage),
    (r"/admin/export", admin.AdminExportData)
]



#!/usr/bin/env python
# coding: utf-8

from urlparse import urlparse
from tornado import testing
from tornado import httpclient
from vanellope import config
from vanellope import app
from tornado import ioloop
import json

# http://www.tornadoweb.org/en/stable/testing.html#tornado.testing.AsyncHTTPTestCase

class TestApiv1(testing.AsyncHTTPTestCase):
    def get_app(self):
        return app.make_app()

    def test_get_api_v1_article_name(self):
        target_url = '/api/v1/article/32ea9174'
        response = self.fetch(target_url)
        parsed_url = urlparse(response.effective_url)
        self.assertEqual(response.code, 200)
        self.assertEqual(parsed_url.path, target_url)
        body = json.loads(response.body)

        expected_keys = [
            u'category',
            u'ext',
            u'uuid',
            u'title',
            u'created_at',
            u'tags',
            u'updated_at',
            u'content',
            u'state'
        ]

        self.assertSequenceEqual(body.keys(), expected_keys);
        self.assertTrue(isinstance(body['category'], unicode));
        self.assertTrue(isinstance(body['state'], unicode));
        self.assertTrue(isinstance(body['uuid'], unicode));
        self.assertTrue(isinstance(body['created_at'], unicode));
        self.assertTrue(isinstance(body['tags'], list));
        self.assertTrue(isinstance(body['content'], unicode));
        self.assertTrue(isinstance(body['ext'], unicode));


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# coding: utf-8

from tornado.testing import unittest
from tornado import gen
from vanellope import database

class TestDatabasePostModel(unittest.TestCase):
    def setUp(self):
        self.posts = database.PostModel()

    def test_get_posts(self):
        posts = self.posts.get_posts()
        self.assertIs(type(posts), list)

    def test_create(self):
        sample = None
        self.assertRaises(TypeError, self.posts.create, sample)

        sample = {}
        self.assertRaises(KeyError, self.posts.create, sample)

        sample = {
            'ext': 'html',
            'title': '1234',
            'content': '1234'
        }



if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# coding: utf-8

import os
from setuptools import setup, find_packages

def read(*paths):
    """Build a file path from *paths* and return the contents."""
    with open(os.path.join(*paths), 'r') as f:
        return f.read()

PACKAGE = "vanellope"
NAME = "vanellope"
DESCRIPTION = "Personal publishing system"
AUTHOR = "Qiao Anran"
AUTHOR_EMAIL = "qiaoanran@gmail.com"
URL = "https://github.com/qar/vanellope"
VERSION = __import__(PACKAGE).__version__

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=read("README.rst"),
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license="MIT",
    url=URL,
    packages=find_packages(exclude=["tests.*", "tests", "test.*", "test"]),

    include_package_data=True,

    install_requires=[
      'tornado',
      'pytz',
      'user_agents',
      'python-dateutil',
    ],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
    ],
    entry_points={
        'console_scripts': [
            'vanellope = vanellope.app:main'
        ]
    },
    zip_safe=True,
)
